#!/usr/bin/env python3
#
#
#

'''
The repro works whether you've got an InfluxDB instance to connect to or not.

The most important thing is that we get a retriable write error - either trying to write
to a non-existing bucket or getting connection refused

'''

import influxdb_client
from datetime import datetime
from influxdb_client.client.write_api import WriteOptions


# Define some config
influxdb_conf = {
    "url" : "http://127.0.0.1:8086",
    "org" : "myorg",
    "token" : ""
}



def handleError(conf: (str, str, str), data: str, exception):
    ''' Dodgy Error handler

    If you call this directly, you'll get an exception
    '''
    print(f"{datetime.now()} Uh-oh, failure")

    # BAD DEVELOPER! This is undefined and will throw
    # an exception, triggering deadlock
    foo = bar

    # Do stuff...



# Instantiate the client
client1 = influxdb_client.InfluxDBClient(
        url = influxdb_conf['url'],
        token = influxdb_conf['token'],
        org = influxdb_conf['org'],
    )

# Write API hook
#
# We set the retries low just to get the repro without
# waiting
write1 = client1.write_api(
        write_options=WriteOptions(
                    batch_size = 5000,
                    flush_interval = 2_000,
                    retry_interval = 1_000,
                    max_retries = 2,
                    max_retry_delay = 2_000
                ),
                error_callback=handleError
)


# Create a point
p = {
        "measurement" : "dummypoint",
        "fields" : {
            "count" : 1
        },
        "time" : datetime.now()
    }

# Try and write it to a non-existent bucket
print("Writing doomed point")
write1.write(
        bucket="reprostats/noexist",
        org="",
        record=p
)

# Try to close the client
write1.close()

'''
The process should now be locked up.

When the retry count was exhausted, the configured error_handler (handleError) was
invoked. Due to a programmer mistake, it threw an exception which killed the
batching thread.

The test made in close() can now never pass, leaving processing trapped in a while loop

'''
print("I'll never get to print")
